/***BSD LICENSE***

Copyright (c) 2009, Zdenek Sikac
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * The name of the author may not be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

***END OF BSD LICENSE***

This program finds hot pixels in dark images and saves their position into file "badpixels" which can be used for dcraw.

USE:

bploc <intensity> <file>

<intensity> 	minimal intensity to be identified as hot pixel
<file>		dark frame 	

Shoot dark frame and do "dcraw -D -4 -j -t 0". Use the resulting PGM file as argument for bploc. The positions of hot pixels are saved into file "badpixels". Rename this file to ".badpixels" and move it to same directory as your raw photos (or any parent directory). The dcraw will use this file automatically to interpolate the hot pixels.*/

#include <iostream>
#include <fstream>
#include <Magick++.h>
#include <string>
#include <sstream>

using namespace std;
using namespace Magick;


//progress indicator
void counter (int c, int cmax, int step) {
int mark = cmax/step;
if ((c%mark) == 0) {cout<<".";cout.flush();};
}

int main(int argc, char *argv[]) {

string image_file = argv[2];
string max_int_string = argv[1];
Image my_image;
int parea,p,intensity,max_int;
PixelPacket *cache;
ofstream my_file("badpixels");

try {
	my_image.read(image_file);
	}

catch ( Exception &error_ ) {
cout << "Caught exception: " << error_.what() << endl;
	}

stringstream(max_int_string)>>max_int;
int const width=my_image.columns();int const height=my_image.rows();
parea = width*height;

cache = my_image.getPixels(0,0,width,height);

cout<<"Searching bad pixels";

my_image.modifyImage();

for (int cx=0;cx<width;cx++) {
	for (int cy=0;cy<height;cy++) {
		p = cy*width+cx;	
		intensity = cache[p].green;
		if (intensity>=max_int) {
			my_file<<cx<<" "<<cy<<" 0\n";
			}
		}
	counter (cx,width,10);
	}
my_image.syncPixels();

cout <<"Done.\n";		

return 0;

}


