#include <iostream>
#include <fstream>
#include <cmath>
#include <string>
#include <iomanip>
#include <algorithm>
#include <sstream>

using namespace std;

const double pi = 3.141592654;

//mathematical functions
template <class T>
T abs (T x) {
T tmp = (x>0)? x:-x;
return (tmp);
}

template <class T>
int sgn (T x) {
int tmp;
if (x==0) tmp = 0;
	else tmp = x / abs (x);
return (tmp);
}

template <class T>
int round (T x) {
int tmp;
tmp = sgn(x)*int(abs(x)+0.5);
return (tmp);
}

template <class T>
T square (T x) {
T tmp = x*x;
return (tmp);
}

template <class T>
float average (T *a, int n) {
float tmp=0;
for (int c=0;c<n;c++) {
	tmp=tmp+*(a+c);
	};
return (tmp/n);
}

template <class T>
float median (T *a, int n) {
sort (a,a+n);
if ((n%2)==0) return *(a+n/2);
return (*(a+(n-1)/2)+*(a+(n+1)/2))/2;
}

template <class T>
float sum (T *a, int n) {
float tmp=0;
for (int c=0;c<n;c++) {
	tmp=tmp+*(a+c);
	};
return (tmp);
}

template <class T>
float scp (T *a, T *b, int n) {
float tmp=0;
for (int c=0;c<n;c++) {
	tmp=tmp+(*(a+c))*(*(b+c));
	};
return (tmp);
}

template <class T>
float stdev (T *a, int n) {
float tmp=0;
float avg=average (a,n);
for (int c=0;c<n;c++) {
	tmp=tmp+square (*(a+c)-avg);
	};
float result = sqrt(tmp/n);
return (result);
}

//average with values outside (median-sigma_lo*stdev, median+sigma_hi*stdev) excluded
template <class T>
float sigma_clip (T *a, int n, float sigma_lo, float sigma_hi) {
int n_true = 0;
float stdev_all = stdev (a, n);
float median_all = quantil (a,n,0.5);
float tmp=0;
T a_cur;

if ((sigma_lo==0)&&(sigma_hi==0)) return (average (a,n));
for (int c=0;c<n;c++) {
	a_cur=*(a+c);	
	if ((a_cur>=(median_all-sigma_lo*stdev_all))&&(a_cur<=(median_all+sigma_hi*stdev_all))) {
		tmp=tmp+*(a+c);
		n_true++;
		}
	}
if (n_true==0) return 0;
return (tmp/n_true);
}

template <class T>
T quantil (T *a, int n, float q) {
T *copy_a;
copy_a = new T [n];
for (int c=0;c<n;c++) {copy_a[c]=a[c];}
if (q<0) q = 0;
if (q>1) q = 1;
nth_element (copy_a,copy_a+int(n*q-1),copy_a+n);
T output = *(copy_a+int(n*q-1));
delete [] copy_a;
return (output);
}

template <class T>
T min (T *a, int n) {
return *min_element(a,a+n);
}

template <class T>
T max (T *a, int n) {
return *max_element(a,a+n);
}

//convert angles to range (-pi,+pi)
template <class T>
T norm_angle (T alpha) { 
T tmp = fmod(alpha,T(2*pi));
if (tmp>T(pi)) tmp = tmp - T(2*pi);
if (tmp<-T(pi)) tmp = tmp + T(2*pi);
return tmp;
}

//matrix class
class Matrix {int m;int n;double *a;
public:
double get_a(int,int);
void set_a(int,int,double);
void swap_rows(int,int);
void separate();
int pos(int,int);
void print();
Matrix (int,int,double*);
};//class Matrix

Matrix::Matrix(int rows,int columns,double *value) {
m = rows; n = columns; int size = m*n;
a = new double [size];
for (int c=0;c<size;c++) {
	a[c]=value[c];
	} //for (int c=0;c<size;c++)
}//Matrix::Matrix(int rows,int columns,double *value)

int Matrix::pos(int row,int column) {
int output = (column-1)+(row-1)*n;
return output;
}//int Matrix::pos(int row,int column)

double Matrix::get_a (int row,int column) {
double output = a[pos(row,column)];
return output;
}//double Matrix::get_a (int row,int column)

void Matrix::set_a (int row, int column, double value) {
a[pos(row,column)] = value;
}//void Matrix::set_a (int row, int column, double value)

void Matrix::swap_rows (int r1,int r2) {
double tmp_a;
for (int c=1;c<=n;c++) {
	tmp_a = get_a(r1,c);
	set_a(r1,c,get_a(r2,c));
	set_a(r2,c,tmp_a);
	}//for (int c=1;c<=n;c++)
}//void Matrix::swap_rows (int r1,int r2)

void Matrix::print() {
for (int r=1;r<=m;r++) {
	for (int c=1;c<=n;c++) {
		cout<<get_a(r,c)<<"\t";
		}//for (int c=1;c<=n;c++)
	cout<<"\n";
	}//for (int r=1;r<=m;r++)
}//void Matrix::print()

void Matrix::separate() {

//zero below diagonal
for (int r1=1;r1<=m;r1++) {
	if (get_a(r1,r1)==0) {
		int swapped_row=r1;
		for (int r3=r1;r3<=m;r3++) {
			if (get_a(r3,r1)!=0) {swapped_row=r3;break;}
			} //for (int r3=r1;r3<=m;r3++)
		swap_rows(r1,swapped_row);
		}//if (get_a(r1,r1)==0)
	for (int r2=r1+1;r2<=m;r2++) {
		if (get_a(r2,r1)==0) continue;
		double multiplier = get_a(r1,r1)/get_a(r2,r1);
		for (int c=1;c<=n;c++) {
			set_a(r2,c,get_a(r2,c)*multiplier);			
			}//for (int c=1;c<=n;c++)
		for (int c=1;c<=n;c++) {
			set_a(r2,c,get_a(r2,c)-get_a(r1,c));			
			}//for (int c=1;c<=n;c++)
		}//for (int r2=r1+1;r2<=m;r2++)
		
	}//for (int r1=1;r1<=m;r1++)

//zero above diagonal
for (int r1=m;r1>=2;r1--) {
	for (int r2=r1-1;r2>=1;r2--) {
		if (get_a(r2,r1)==0) continue;
		double multiplier = get_a(r1,r1)/get_a(r2,r1);
		for (int c=1;c<=n;c++) {
			set_a(r2,c,get_a(r2,c)*multiplier);			
			}//for (int c=1;c<=n;c++)
		for (int c=1;c<=n;c++) {
			set_a(r2,c,get_a(r2,c)-get_a(r1,c));			
			}//for (int c=1;c<=n;c++)
		}//for (int r2=r1-1;r2>=1;r2--)
	}//for (int r1=m;r1>=2;r1--)

//set diagonal to 1
for (int r=1;r<=m;r++) {
	double multiplier = 1/get_a(r,r);
	for (int c=1;c<=n;c++) {
		set_a(r,c,get_a(r,c)*multiplier);			
		}//for (int c=1;c<=n;c++)
	}//for (int r=1;r<=m;r++)
}//void Matrix::separate()

class LinearRegression {
double cx1,cx2,cx3,cx4,b;
int n;
double sd_res,avg_res;
double *one,*res;

public:
LinearRegression (double*,double*,double*,double*,double*,int);
double get_b(),get_cx1(),get_cx2(),get_cx3(),get_cx4(),get_sd_res(),get_avg_res();
~LinearRegression ();
};//class LinearRegression

double LinearRegression::get_b() {return(b);}
double LinearRegression::get_cx1() {return(cx1);}
double LinearRegression::get_cx2() {return(cx2);}
double LinearRegression::get_cx3() {return(cx3);}
double LinearRegression::get_cx4() {return(cx4);}
double LinearRegression::get_sd_res() {return(sd_res);}
double LinearRegression::get_avg_res() {return(avg_res);}

LinearRegression::LinearRegression (double *y,double *x1,double *x2,double *x3,double *x4,int ni) {
n=ni;
one = new double[n];
res = new double[n];

for (int c=0;c<n;c++) {
	one[c] = 1;
	}//for (int c=0;c<n;c++)

double a[] = {
scp(x4,x4,n),scp(x3,x4,n),scp(x2,x4,n),scp(x1,x4,n),scp(one,x4,n),scp(y,x4,n),
scp(x4,x3,n),scp(x3,x3,n),scp(x2,x3,n),scp(x1,x3,n),scp(one,x3,n),scp(y,x3,n),
scp(x4,x2,n),scp(x3,x2,n),scp(x2,x2,n),scp(x1,x2,n),scp(one,x2,n),scp(y,x2,n),
scp(x4,x1,n),scp(x3,x1,n),scp(x2,x1,n),scp(x1,x1,n),scp(one,x1,n),scp(y,x1,n),
scp(x4,one,n),scp(x3,one,n),scp(x2,one,n),scp(x1,one,n),scp(one,one,n),scp(y,one,n)
};

Matrix reg (5,6,a);
reg.separate();

cx4=reg.get_a(1,6);
cx3=reg.get_a(2,6);
cx2=reg.get_a(3,6);
cx1=reg.get_a(4,6);
b=reg.get_a(5,6);

for (int c=0;c<n;c++) {
	res[c] = y[c]-b-cx1*x1[c]-cx2*x2[c]-cx3*x3[c]-cx4*x4[c];
	}//for (int c=0;c<n;c++)

avg_res = average (res,n);
sd_res = stdev (res,n);
};//LinearRegression::LinearRegression (double *y,double *x1,double *x2,double *x3,double *x4,int n)

LinearRegression::~LinearRegression() {
delete[] one;
delete[] res;
};//LinearRegression::~LinearRegression()



//progress indicator
void counter (int c, int cmax, int step) {
int mark = cmax/step;
if (mark<1) return;
if ((c%mark) == 0) {cout<<"."<<flush;};
}//void counter (int c, int cmax, int step)

void error_message (int return_value) {
if (not (return_value == 0)) {
	cout<<"Error value "<<return_value<<" returned.\n";};
}//void error_message (int return_value)

//definition of vector and its operators
template <class T>
class Vector {
T x1,x2;
public:
float size();
Vector (T x1i,T x2i) {x1 = x1i; x2 = x2i;};
T get_x1() {return(x1);};
T get_x2() {return(x2);};
};

template <class T>
float Vector<T>::size() {
return (sqrt(square(x1)+square(x2)));
}

template <class T>
Vector <T> add (Vector <T> a, Vector <T> b) {
Vector <T> tmp (0,0);
tmp.x1 = a.get_x1()+b.get_x1();
tmp.x2 = a.get_x2()+b.get_x2();
return (tmp);
}

template <class T>
Vector <T> subtract (Vector <T> a, Vector <T> b) {
Vector <T> tmp (a.get_x1()-b.get_x1(),a.get_x2()-b.get_x2());
return (tmp);
}

