/***BSD LICENSE***

Copyright (c) 2010, Zdenek Sikac
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * The name of the author may not be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

***END OF BSD LICENSE***


version 0.1

remgrad removes gradients from calibrated deep sky images to reduce effect of light pollution.

USE:

remgrad <SWITCHES> <input_file> <output_file>

SWITCHES:

-n [int]	number of segments for calculation of gradient
-R		remove gradient from red channel
-G		remove gradient from green channel
-B		remove gradient from blue channel
*/

#include <iostream>
#include <algorithm>
#include <Magick++.h>
#include "math.h"

using namespace std;
using namespace Magick;

const int n_seg_default = 9;

class ImageSample {
float x,y,x2,y2;//coordinates of central point for each image segment
float i;//intensity
float i_res;//residues from regression
bool valid;

public:
void set(float,float,float,bool);void set_valid(bool);void set_residue (float);
float get_x(),get_y(),get_x2(),get_y2(),get_i(),get_i_res();
bool get_valid();
};//class ImageSample

void ImageSample::set (float xi,float yi,float ii,bool validi) {
x=xi;y=yi;i=ii;valid=validi;x2=square(xi);y2=square(yi);
};//void ImageSample::set (float xi,float yi,float ii,bool validi)

void ImageSample::set_valid (bool validi) {
valid=validi;
};//void ImageSample::set_valid (bool validi)

void ImageSample::set_residue (float i_res_i) {
i_res=i_res_i;
};//void ImageSample::set_residue (float i_res_i)

float ImageSample::get_x() {return(x);}
float ImageSample::get_y() {return(y);}
float ImageSample::get_x2() {return(x2);}
float ImageSample::get_y2() {return(y2);}
float ImageSample::get_i() {return(i);}
float ImageSample::get_i_res() {return(i_res);}
bool ImageSample::get_valid() {return(valid);}

class Workspace {
Image image_in; int n_seg,width,height,area,px,py,seg_area; Image image_out;
float R_c1,R_cx,R_cy,R_cx2,R_cy2,R_avg_res,R_sd_res;
float G_c1,G_cx,G_cy,G_cx2,G_cy2,G_avg_res,G_sd_res;
float B_c1,B_cx,B_cy,B_cx2,B_cy2,B_avg_res,B_sd_res;

public:
void set(Image,int);
void calculate_gradients();
void remove_gradient_R();void remove_gradient_G();void remove_gradient_B();
void print();void save_result(string);
};//class Workspace

void Workspace::set(Image in, int n) {
n_seg=n;image_in=in;image_out=image_in;
width=image_in.columns();height=image_in.rows();area=width*height;
px=width/n_seg; py=height/n_seg; seg_area=px*py;
}//void Workspace::set(Image in, int n)

void Workspace::calculate_gradients() {

cout<<"Processing area size: px = "<<px<<", py = "<<py<<", area = "<<seg_area<<"\n";
//cout<<"x_mid\ty_mid\tmed_int\n";

PixelPacket *cache;int *R,*G,*B;float R_median,G_median,B_median;

int ns=square(n_seg);
ImageSample *R_sample,*G_sample,*B_sample;
R_sample = new ImageSample[ns];
G_sample = new ImageSample[ns];
B_sample = new ImageSample[ns];

image_in.modifyImage();

int c1=0;
for (int cx=0;cx<n_seg;cx++) {
	for (int cy=0;cy<n_seg;cy++) {
		float x_mid = px*(cx+0.5);		
		float y_mid = py*(cy+0.5);
		cache = image_in.getPixels(cx*px,cy*py,px,py); //cache of input image
		R = new int [seg_area];
		G = new int [seg_area];
		B = new int [seg_area];
		for (int c2=0;c2<seg_area;c2++) {
			R[c2] = cache[c2].red;
			G[c2] = cache[c2].green;
			B[c2] = cache[c2].blue;
			} //for (int c2=0;c2<seg_area;c2++)
		R_median = quantil (R, seg_area, 0.5);R_sample[c1].set(x_mid,y_mid,R_median,true);
		G_median = quantil (G, seg_area, 0.5);G_sample[c1].set(x_mid,y_mid,G_median,true);
		B_median = quantil (B, seg_area, 0.5);B_sample[c1].set(x_mid,y_mid,B_median,true);
		c1++;
		delete[] R;		
		delete[] G;		
		delete[] B;		
		}//for (int cy=0;cy<n_seg;cy++)
	}//for (int cx=0;cx<n_seg;cx++)
image_in.syncPixels(); //pixel cache is synchronized with input image		
 
double *x,*y,*x2,*y2,*Rs,*Gs,*Bs;
x = new double[ns];
y = new double[ns];
x2 = new double[ns];
y2 = new double[ns];
Rs = new double[ns];
Gs = new double[ns];
Bs = new double[ns];

for (int c=0;c<ns;c++) {
	x[c] = G_sample[c].get_x();
	y[c] = G_sample[c].get_y();
	x2[c] = G_sample[c].get_x2();
	y2[c] = G_sample[c].get_y2();
	Rs[c] = R_sample[c].get_i();
	Gs[c] = G_sample[c].get_i();
	Bs[c] = B_sample[c].get_i();
	}//for (int c=0;c<ns;c++)

LinearRegression R_lr (Rs,x,y,x2,y2,ns);
LinearRegression G_lr (Gs,x,y,x2,y2,ns);
LinearRegression B_lr (Bs,x,y,x2,y2,ns);

R_c1=R_lr.get_b();R_cx=R_lr.get_cx1();R_cy=R_lr.get_cx2();R_cx2=R_lr.get_cx3();R_cy2=R_lr.get_cx4();
R_avg_res=R_lr.get_avg_res();R_sd_res=R_lr.get_sd_res();
G_c1=G_lr.get_b();G_cx=G_lr.get_cx1();G_cy=G_lr.get_cx2();G_cx2=G_lr.get_cx3();G_cy2=G_lr.get_cx4();
G_avg_res=G_lr.get_avg_res();G_sd_res=G_lr.get_sd_res();
B_c1=B_lr.get_b();B_cx=B_lr.get_cx1();B_cy=B_lr.get_cx2();B_cx2=B_lr.get_cx3();B_cy2=B_lr.get_cx4();
B_avg_res=B_lr.get_avg_res();B_sd_res=B_lr.get_sd_res();
}//void Workspace::calculate_gradients()

void Workspace::print() {
cout<<"R:\tc1="<<R_c1<<" cx="<<R_cx<<" cy="<<R_cy<<" cx2="<<R_cx2<<" cy2="<<R_cy2<<" avgres="<<R_avg_res<<" sdres="<<R_sd_res<<"\n";
cout<<"G:\tc1="<<G_c1<<" cx="<<G_cx<<" cy="<<G_cy<<" cx2="<<G_cx2<<" cy2="<<G_cy2<<" avgres="<<G_avg_res<<" sdres="<<G_sd_res<<"\n";
cout<<"B:\tc1="<<B_c1<<" cx="<<B_cx<<" cy="<<B_cy<<" cx2="<<B_cx2<<" cy2="<<B_cy2<<" avgres="<<B_avg_res<<" sdres="<<B_sd_res<<"\n";
}//void Workspace::print()

void Workspace::remove_gradient_R() {
int in,out;
PixelPacket *cache;
cache=image_out.getPixels(0,0,width,height);
image_out.modifyImage();
for (int x=0;x<width;x++) {
	for (int y=0;y<height;y++) {
		in=cache[x+y*width].red;
		out=in-R_cx*x-R_cy*y-R_cx2*square(x)-R_cy2*square(y);
		if (out<0) out=0;
		if (out>(signed)MaxRGB) out=MaxRGB;
		cache[x+y*width].red=out;
	}//for (int y=0;y<height;y++)
}//for (int x=0;x<width;x++)
image_out.syncPixels();
}//void Workspace::remove_gradient_R() 

void Workspace::remove_gradient_G() {
int in,out;
PixelPacket *cache;
cache=image_out.getPixels(0,0,width,height);
image_out.modifyImage();
for (int x=0;x<width;x++) {
	for (int y=0;y<height;y++) {
		in=cache[x+y*width].green;
		out=in-G_cx*x-G_cy*y-G_cx2*square(x)-G_cy2*square(y);
		if (out<0) out=0;
		if (out>(signed)MaxRGB) out=MaxRGB;
		cache[x+y*width].green=out;
	}//for (int y=0;y<height;y++)
}//for (int x=0;x<width;x++)
image_out.syncPixels();
}//void Workspace::remove_gradient_G() 

void Workspace::remove_gradient_B() {
int in,out;
PixelPacket *cache;
cache=image_out.getPixels(0,0,width,height);
image_out.modifyImage();
for (int x=0;x<width;x++) {
	for (int y=0;y<height;y++) {
		in=cache[x+y*width].blue;
		out=in-B_cx*x-B_cy*y-B_cx2*square(x)-B_cy2*square(y);
		if (out<0) out=0;
		if (out>(signed)MaxRGB) out=MaxRGB;
		cache[x+y*width].blue=out;
	}//for (int y=0;y<height;y++)
}//for (int x=0;x<width;x++)
image_out.syncPixels();
}//void Workspace::remove_gradient_B() 

void Workspace::save_result(string filename) {
image_out.write(filename);
}//void Workspace::save_result(string filename)

int main(int argc, char *argv[]) {

string in_file, out_file;
Image in_image,out_image;
//assign filenames

int n_seg = n_seg_default;bool remove_R=false,remove_G=false,remove_B=false;

int input_image_pos = 1;// search for input image
string option, parameter1;//strings to be read from shell

for (int c=1;c<argc-2;c++) {
	option = argv[c];
	parameter1 = argv[c+1];
	if (option =="-R") {remove_R=true;input_image_pos+=1;continue;}
	if (option =="-G") {remove_G=true;input_image_pos+=1;continue;}
	if (option =="-B") {remove_B=true;input_image_pos+=1;continue;}
	if (option =="-n") {stringstream(parameter1)>>n_seg;input_image_pos+=2;continue;}
	}//for (int c=1;c<argc-2;c++)	

in_file = argv[input_image_pos];
out_file = argv[input_image_pos+1];

cout<<in_file<<"\t"<<out_file<<"\n";

//open image
try {
	in_image.read(in_file);
	}

catch ( Exception &error_ ) {
cout << "Caught exception: " << error_.what() << endl;
	}

out_image=in_image;

Workspace ws;
ws.set(in_image,n_seg);
ws.calculate_gradients();
ws.print();
if(remove_R) ws.remove_gradient_R();
if(remove_G) ws.remove_gradient_G();
if(remove_B) ws.remove_gradient_B();
ws.save_result(out_file);
return 0;
}

