/***BSD LICENSE***

Copyright (c) 2009, Zdenek Sikac
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * The name of the author may not be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

***END OF BSD LICENSE***


version 0.1

remlp adjusts intensities of color channels in a deep sky image to reduce effect of light pollution. The input image needs to be white balanced.

USE:

remlp <input_file> <output_file>

*/

#include <iostream>
#include <algorithm>
#include <Magick++.h>

using namespace std;
using namespace Magick;

template <class T>
T quantil (T *a, int n, float q) {
T *copy_a;
copy_a = new T [n];
for (int c=0;c<n;c++) {copy_a[c]=a[c];}
if (q<0) q = 0;
if (q>1) q = 1;
nth_element (copy_a,copy_a+int(n*q-1),copy_a+n);
T output = *(copy_a+int(n*q-1));
delete [] copy_a;
return (output);
}

template <class T>
T min (T *a, int n) {
return *min_element(a,a+n);
}

//progress indicator
void counter (int c, int cmax, int step) {
int mark = cmax/step;
if ((c%mark) == 0) {cout<<".";cout.flush();};
}

int main(int argc, char *argv[]) {

string in_file, out_file;
Image p_image;
PixelPacket *cache;

int *in_R, *in_G,*in_B;
int *out_R,*out_G,*out_B;

//assign filenames

in_file = argv[1];
out_file = argv[2];

//open image
try {
	p_image.read(in_file);
	}

catch ( Exception &error_ ) {
cout << "Caught exception: " << error_.what() << endl;
	}

int const width=p_image.columns();int const height=p_image.rows(); int const area = width*height;

in_R = new int [area]; in_G = new int [area]; in_B = new int [area];
out_R = new int [area]; out_G = new int [area]; out_B = new int [area];

p_image.modifyImage();
cache = p_image.getPixels(0,0,width,height);
for (int c=0;c<area;c++) {
	in_R[c] = cache[c].red;
	in_G[c] = cache[c].green;
	in_B[c] = cache[c].blue;
	} //for (int c=0;c<area;c++)

int median_R = quantil(in_R,area,0.5);
int median_G = quantil(in_G,area,0.5);
int median_B = quantil(in_B,area,0.5);
int median[3] = {median_R,median_G,median_B}; 
int median_min = min(median,3);

cout<<"Red median: "<<median_R<<"\n";
cout<<"Green median: "<<median_G<<"\n";
cout<<"Blue median: "<<median_B<<"\n";

cout<<"Adjusting levels";
for (int c=0;c<area;c++) {
	out_R[c] = in_R[c]-median_R+median_min;
	out_G[c] = in_G[c]-median_G+median_min;
	out_B[c] = in_B[c]-median_B+median_min;

	if (out_R[c]<0) out_R[c]=0;
	if (out_G[c]<0) out_G[c]=0;
	if (out_B[c]<0) out_B[c]=0;

	if (out_R[c]> (signed) MaxRGB) out_R[c]=MaxRGB;
	if (out_G[c]> (signed) MaxRGB) out_G[c]=MaxRGB;
	if (out_B[c]> (signed) MaxRGB) out_B[c]=MaxRGB;

	cache[c]=Color(out_R[c],out_G[c],out_B[c]);
	counter (c, area, 10);
	} //for (int c=0;c<area;c++)
cout<<"Done.\n";

p_image.syncPixels();
p_image.write(out_file);	

return 0;
}



